/**
 * Common types, conversion functions and Multikey conversion utilities for the rest of the code.
 *
 * Licensed by under the W3C Software and Document License, https://www.w3.org/Consortium/Legal/copyright-software.
 * Copyright ©2024 World Wide Web Consortium. https://www.w3.org/copyright/software-license-2023/
 *
 * @module
 */
import * as eddsa from "./eddsa.js";
import * as ecdsa from "./ecdsa.js";
/************************************************************************* */
/* Values to handle the various preamble bytes for the different key types */
/************************************************************************* */
/**
 * Names for the various crypto curves
 */
export var CryptoCurves;
(function (CryptoCurves) {
    CryptoCurves["ECDSA_384"] = "secp384r1";
    CryptoCurves["ECDSA_256"] = "secp256r1";
    CryptoCurves["EDDSA"] = "ed25519";
})(CryptoCurves || (CryptoCurves = {}));
/**
 * Names for the key types
 */
export var CryptoKeyTypes;
(function (CryptoKeyTypes) {
    CryptoKeyTypes["PUBLIC"] = "public";
    CryptoKeyTypes["SECRET"] = "secret";
})(CryptoKeyTypes || (CryptoKeyTypes = {}));
/**
 * Preamble value for EDDSA, a.k.a. `ed25519` curve
 */
export const EddsaPreambles = {
    public: [0xed, 0x01],
    secret: [0x80, 0x26],
};
/**
 * Preamble for ECDSA `P-256`, a.k.a. `secp256r1` curve
 */
export const Ecdsa256Preambles = {
    public: [0x80, 0x24],
    secret: [0x86, 0x26],
};
/**
 * Preamble for ECDSA `P-384`, a.k.a. `secp384r1` curve
 */
export const Ecdsa384Preambles = {
    public: [0x81, 0x24],
    secret: [0x87, 0x26],
};
/**
 * What preambles must be used for a Curve?
 */
export const classToPreamble = {
    [CryptoCurves.EDDSA]: EddsaPreambles,
    [CryptoCurves.ECDSA_256]: Ecdsa256Preambles,
    [CryptoCurves.ECDSA_384]: Ecdsa384Preambles,
};
/**
 * What coder function must be used to convert from Multikey to JWK (data)?
 */
export const classToDecoder = {
    [CryptoCurves.EDDSA]: eddsa.multikeyBinaryToJWK,
    [CryptoCurves.ECDSA_256]: ecdsa.multikeyBinaryToJWK,
    [CryptoCurves.ECDSA_384]: ecdsa.multikeyBinaryToJWK,
};
/**
 * What coder function must be used to convert from JWK to Multikey?
 */
export const classToEncoder = {
    [CryptoCurves.EDDSA]: eddsa.JWKToMultikeyBinary,
    [CryptoCurves.ECDSA_256]: ecdsa.JWKToMultikeyBinary,
    [CryptoCurves.ECDSA_384]: ecdsa.JWKToMultikeyBinary,
};
/**
 * List of possible ECDSA Curves. Having this here declaratively may make it easier if
 * in the future, a new curve is added to the family (P-512)?
 */
export const ECDSACurves = [CryptoCurves.ECDSA_256, CryptoCurves.ECDSA_384];
/**
 * Classify the crypto key based on the multikey preamble characters that are at the start of the code.
 * These are two binary numbers, signalling the crypto category (`ecdsa` or `eddsa`) and, in the former case,
 * the additional reference to the exact curve.
 *
 * @param preamble
 * @returns
 */
export function preambleToCryptoData(preamble) {
    // Ugly but effective and simple trick to compare two arrays
    const eq = (a, b) => JSON.stringify(a) === JSON.stringify(b);
    if (preamble.length !== 2) {
        throw new Error(`${preamble} is not valid, it should have a size of exactly 2.`);
    }
    // The real classification...
    if (eq(preamble, Ecdsa256Preambles.secret)) {
        return {
            crCurve: CryptoCurves.ECDSA_256,
            crType: CryptoKeyTypes.SECRET,
        };
    }
    else if (eq(preamble, Ecdsa256Preambles.public)) {
        return {
            crCurve: CryptoCurves.ECDSA_256,
            crType: CryptoKeyTypes.PUBLIC,
        };
    }
    else if (eq(preamble, Ecdsa384Preambles.secret)) {
        return {
            crCurve: CryptoCurves.ECDSA_384,
            crType: CryptoKeyTypes.SECRET,
        };
    }
    else if (eq(preamble, Ecdsa384Preambles.public)) {
        return {
            crCurve: CryptoCurves.ECDSA_384,
            crType: CryptoKeyTypes.PUBLIC,
        };
    }
    else if (eq(preamble, EddsaPreambles.secret)) {
        return {
            crCurve: CryptoCurves.EDDSA,
            crType: CryptoKeyTypes.SECRET,
        };
    }
    else if (eq(preamble, EddsaPreambles.public)) {
        return {
            crCurve: CryptoCurves.EDDSA,
            crType: CryptoKeyTypes.PUBLIC,
        };
    }
    else {
        throw new Error(`${preamble} is unknown. Should refer to secret or private eddsa or ecdsa (the latter with P-256 or P-384)`);
    }
}
