"use strict";
/**
 * Conversion to and from [Multikey format](https://www.w3.org/TR/controller-document/#multikey) from
 * JWK or WebCrypto for the three EC curves that are defined for Verifiable Credentials: [ECDSA with P-256 and P-384](https://www.w3.org/TR/vc-di-ecdsa/#multikey)
 * and [EDDSA](https://www.w3.org/TR/vc-di-eddsa/#multikey).
 *
 * Licensed by under the W3C Software and Document License, https://www.w3.org/Consortium/Legal/copyright-software.
 * Copyright ©2024 World Wide Web Consortium. https://www.w3.org/copyright/software-license-2023/
 *
 * @package
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.multikeyToJWK = multikeyToJWK;
exports.multikeyToCrypto = multikeyToCrypto;
exports.JWKToMultikey = JWKToMultikey;
exports.cryptoToMultikey = cryptoToMultikey;
const convert = __importStar(require("./lib/convert.js"));
// This type guard function is reused at two different places, better factor it out...
// deno-lint-ignore no-explicit-any
function isMultikeyPair(obj) {
    return obj.publicKeyMultibase !== undefined;
}
function multikeyToJWK(keys) {
    const input = isMultikeyPair(keys) ? keys : { publicKeyMultibase: keys };
    const jwk_keys = convert.multikeyToJWK(input);
    if (isMultikeyPair(keys)) {
        return jwk_keys;
    }
    else {
        return jwk_keys.publicKey;
    }
}
async function multikeyToCrypto(keys) {
    const input = isMultikeyPair(keys) ? keys : { publicKeyMultibase: keys };
    const jwkPair = multikeyToJWK(input);
    const algorithm = { name: "" };
    // We have to establish what the algorithm type is from the public jwk
    switch (jwkPair.publicKey.kty) {
        case 'EC':
            algorithm.name = "ECDSA";
            algorithm.namedCurve = jwkPair.publicKey.crv;
            break;
        case 'OKP':
            algorithm.name = "Ed25519";
            break;
        default:
            // In fact, this does not happen; the JWK comes from our own
            // generation, that raises an error earlier in this case.
            // But this keeps typescript happy...
            throw new Error("Unknown kty value for the JWK key");
    }
    const publicKey = await crypto.subtle.importKey("jwk", jwkPair.publicKey, algorithm, true, ["verify"]);
    if (isMultikeyPair(keys)) {
        if (jwkPair.privateKey !== undefined) {
            const privateKey = await crypto.subtle.importKey("jwk", jwkPair.privateKey, algorithm, true, ["sign"]);
            return { publicKey, privateKey };
        }
        else {
            throw new Error("Unknown privateKey for the JWK key; something went wrong");
        }
    }
    else {
        return publicKey;
    }
}
// Implementation of the overloaded functions
function JWKToMultikey(keys) {
    // deno-lint-ignore no-explicit-any
    function isJWKKeyPair(obj) {
        return obj.publicKey !== undefined;
    }
    const input = isJWKKeyPair(keys) ? keys : { publicKey: keys };
    const m_keys = convert.JWKToMultikey(input);
    if (isJWKKeyPair(keys)) {
        return m_keys;
    }
    else {
        return m_keys.publicKeyMultibase;
    }
}
async function cryptoToMultikey(keys) {
    // deno-lint-ignore no-explicit-any
    function isCryptoKeyPair(obj) {
        return obj.publicKey !== undefined;
    }
    const isPair = isCryptoKeyPair(keys);
    const publicKeyCr = isPair ? keys.publicKey : keys;
    const privateKeyCr = isPair ? keys.privateKey : undefined;
    // Generate the JWK version of the cryptokeys: 
    const jwkKeyPair = {
        publicKey: await crypto.subtle.exportKey("jwk", publicKeyCr),
        privateKey: (isPair && privateKeyCr !== undefined) ? await crypto.subtle.exportKey("jwk", privateKeyCr) : undefined,
    };
    // Ready for conversion
    const output = JWKToMultikey(jwkKeyPair);
    // Return the right version
    if (isPair) {
        return output;
    }
    else {
        return output.publicKeyMultibase;
    }
}
